#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Filter::Rule::OperationBinary class and it`s subclassess tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
# Comments:
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 15;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Filter::Rule::OperationBinary';

        use_ok('Tester');
        use_ok('Data::Filter::Rule');
        use_ok('Data::Filter::Rule::Operation');
        use_ok('Data::Core');
        use_ok($TESTED_CLASS);
        use_ok('Data::Filter::Rule::Variable');
        use_ok('Data::Filter::Rule::Constant');
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Data::Filter::Rule::OperationBinary::VERSION) if Tester::D();
diag("Data::Filter::Rule::Variable version: " . $Data::Filter::Rule::Variable::VERSION) if Tester::D();
diag("Data::Filter::Rule::Constant version: " . $Data::Filter::Rule::Constant::VERSION) if Tester::D();

# Prepare the operands
my $op1 = Data::Filter::Rule::Constant->new('constant', 'constant_value');
isa_ok($op1, 'Data::Filter::Rule::Constant');
my $op2 = Data::Filter::Rule::Variable->new('variable', 'Node/Name');
isa_ok($op2, 'Data::Filter::Rule::Variable');

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new('and', $op1, $op2);
isa_ok($instance, $TESTED_CLASS);

# Test the rule interface
is($instance->value(),     undef,                              "Rule interface test 01");
is($instance->datatype(),  'BOOL',                             "Rule interface test 02");
is($instance->operation(), 'AND',                              "Rule interface test 03");
is($instance->name(),      'OperationBinary',                  "Rule interface test 04");
is($instance->to_string(), '("constant_value" AND Node/Name)', "Rule interface test 05");

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();
