#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Query::Object class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
# Comments:
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 21;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Query::Object';

        use_ok('Tester');
        use_ok('Data::Filter::Traverser');
        use_ok('Data::Query');
        use_ok('Data::Tree');
        use_ok('Data::Filter::Factory');
        use_ok('Mentat::Message::Value::IDEA');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Data::Query::Object::VERSION) if Tester::D();

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new(dt_resolver => Mentat::Message::Value::IDEA->new());
isa_ok($instance, $TESTED_CLASS);

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();

# Filter factory instance
my $ffactory = Data::Filter::Factory->new_from_file('../../conf/filter-grammar.conf');
isa_ok($ffactory, 'Data::Filter::Factory');

# View the created instance
diag("Instance dump 02: " . Dumper($ffactory)) if Tester::D();

#-------------------------------------------------------------------------------

my $object1 = {
        'ID' => "1234",
        'Source' => {
            'Ident' => "a1b2c3d4",
            'IP4' => "195.113.144.194",
        },
        'Target' => {
            'Ident' => "d1c2b3a4",
            'IP4' => "195.113.144.195",
        },
        'Reference' => 125,
    };

#-------------------------------------------------------------------------------

my @FILTERS = (
        ['? ID',1],
        ['? Nothing',0],
        ['ID == "1234" and Target/Ident eq "d1c2b3a4"',1],
        ['ID < "1234" and Target/Ident eq "d1c2b3a4"',0],
        ['ID > "1234" and Target/Ident eq "d1c2b3a4"',0],
        ['(ID eq "1234" and Target/Ident eq "d1c2b3a4") || Reference == "125"',1],
        ['Source/Ident eq "a1b2c3d5"',0],
        ['Source/Ident eq "a1b2c3d4"',1],
        ['Source/Ident in ["A1b2c3d4","a1B2c3d4","a1b2C3d4","a1b2c3d4"]',1],
        ['Source/IP4 in [192.0.2.51,192.0.2.52,195.113.144.194,192.0.2.50]',1],
        ['Source/Ident like "^a\db"',1],
        ['Source/Ident like "^a\d\db"',0],

    );

# Create and test couple of filters
my $filter;
my $counter = 0;
foreach my $rule (@FILTERS)
{
    ++$counter;
    $filter = $ffactory->filter_from_string($rule->[0]);
    is($instance->is_satisfied_by($filter, $object1), $rule->[1], "Filter test $counter: $rule->[0]");
}
